# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types.skynet import SimpleResp
from nextbillion_sdk.types.geofence import (
    BatchListResponse,
    BatchCreateResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBatch:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: NextbillionSDK) -> None:
        batch = client.geofence.batch.create(
            key="key=API_KEY",
        )
        assert_matches_type(BatchCreateResponse, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: NextbillionSDK) -> None:
        batch = client.geofence.batch.create(
            key="key=API_KEY",
            geofences=[
                {
                    "type": "`circle`",
                    "circle": {
                        "center": {
                            "lat": 0,
                            "lon": 0,
                        },
                        "radius": 0,
                    },
                    "custom_id": "custom_id",
                    "isochrone": {
                        "coordinates": '"coordinates": "13.25805884,77.91083661"',
                        "contours_meter": 0,
                        "contours_minute": 0,
                        "denoise": 0,
                        "departure_time": 0,
                        "mode": "`car`",
                    },
                    "meta_data": '{\n  "country": "USA",\n  "state": "California"\n}',
                    "name": '"name":"Los Angeles Downtown"',
                    "polygon": {
                        "geojson": {
                            "coordinates": [[0]],
                            "type": "type",
                        }
                    },
                    "tags": ['"tags":["tags_1", "O69Am2Y4KL8q5Y5JuD-Fy-tdtEpkTRQo_ZYIK7"]'],
                }
            ],
        )
        assert_matches_type(BatchCreateResponse, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: NextbillionSDK) -> None:
        response = client.geofence.batch.with_raw_response.create(
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        batch = response.parse()
        assert_matches_type(BatchCreateResponse, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: NextbillionSDK) -> None:
        with client.geofence.batch.with_streaming_response.create(
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            batch = response.parse()
            assert_matches_type(BatchCreateResponse, batch, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: NextbillionSDK) -> None:
        batch = client.geofence.batch.list(
            ids="ids",
            key="key=API_KEY",
        )
        assert_matches_type(BatchListResponse, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: NextbillionSDK) -> None:
        response = client.geofence.batch.with_raw_response.list(
            ids="ids",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        batch = response.parse()
        assert_matches_type(BatchListResponse, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: NextbillionSDK) -> None:
        with client.geofence.batch.with_streaming_response.list(
            ids="ids",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            batch = response.parse()
            assert_matches_type(BatchListResponse, batch, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: NextbillionSDK) -> None:
        batch = client.geofence.batch.delete(
            key="key=API_KEY",
        )
        assert_matches_type(SimpleResp, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_delete_with_all_params(self, client: NextbillionSDK) -> None:
        batch = client.geofence.batch.delete(
            key="key=API_KEY",
            ids=["string"],
        )
        assert_matches_type(SimpleResp, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: NextbillionSDK) -> None:
        response = client.geofence.batch.with_raw_response.delete(
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        batch = response.parse()
        assert_matches_type(SimpleResp, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: NextbillionSDK) -> None:
        with client.geofence.batch.with_streaming_response.delete(
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            batch = response.parse()
            assert_matches_type(SimpleResp, batch, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncBatch:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncNextbillionSDK) -> None:
        batch = await async_client.geofence.batch.create(
            key="key=API_KEY",
        )
        assert_matches_type(BatchCreateResponse, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        batch = await async_client.geofence.batch.create(
            key="key=API_KEY",
            geofences=[
                {
                    "type": "`circle`",
                    "circle": {
                        "center": {
                            "lat": 0,
                            "lon": 0,
                        },
                        "radius": 0,
                    },
                    "custom_id": "custom_id",
                    "isochrone": {
                        "coordinates": '"coordinates": "13.25805884,77.91083661"',
                        "contours_meter": 0,
                        "contours_minute": 0,
                        "denoise": 0,
                        "departure_time": 0,
                        "mode": "`car`",
                    },
                    "meta_data": '{\n  "country": "USA",\n  "state": "California"\n}',
                    "name": '"name":"Los Angeles Downtown"',
                    "polygon": {
                        "geojson": {
                            "coordinates": [[0]],
                            "type": "type",
                        }
                    },
                    "tags": ['"tags":["tags_1", "O69Am2Y4KL8q5Y5JuD-Fy-tdtEpkTRQo_ZYIK7"]'],
                }
            ],
        )
        assert_matches_type(BatchCreateResponse, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.geofence.batch.with_raw_response.create(
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        batch = await response.parse()
        assert_matches_type(BatchCreateResponse, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.geofence.batch.with_streaming_response.create(
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            batch = await response.parse()
            assert_matches_type(BatchCreateResponse, batch, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncNextbillionSDK) -> None:
        batch = await async_client.geofence.batch.list(
            ids="ids",
            key="key=API_KEY",
        )
        assert_matches_type(BatchListResponse, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.geofence.batch.with_raw_response.list(
            ids="ids",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        batch = await response.parse()
        assert_matches_type(BatchListResponse, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.geofence.batch.with_streaming_response.list(
            ids="ids",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            batch = await response.parse()
            assert_matches_type(BatchListResponse, batch, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncNextbillionSDK) -> None:
        batch = await async_client.geofence.batch.delete(
            key="key=API_KEY",
        )
        assert_matches_type(SimpleResp, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        batch = await async_client.geofence.batch.delete(
            key="key=API_KEY",
            ids=["string"],
        )
        assert_matches_type(SimpleResp, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.geofence.batch.with_raw_response.delete(
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        batch = await response.parse()
        assert_matches_type(SimpleResp, batch, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.geofence.batch.with_streaming_response.delete(
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            batch = await response.parse()
            assert_matches_type(SimpleResp, batch, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .polygon_geojson import PolygonGeojson

__all__ = ["Geofence", "CircleCenter"]


class CircleCenter(BaseModel):
    lat: Optional[float] = None
    """Latitude of the location."""

    lon: Optional[float] = None
    """Longitude of the location."""


class Geofence(BaseModel):
    id: Optional[str] = None
    """ID of the geofence provided/generated at the time of creating the geofence."""

    circle_center: Optional[CircleCenter] = None

    circle_radius: Optional[float] = None
    """
    When the `type` of the geofence is `circle`, this property returns the radius of
    the geofence in meters (m).
    """

    created_at: Optional[int] = None
    """
    Time at which the geofence was created, expressed as a UNIX timestamp in
    seconds.
    """

    geojson: Optional[PolygonGeojson] = None
    """An object with geoJSON details of the geofence.

    The contents of this object follow the
    [geoJSON standard](https://datatracker.ietf.org/doc/html/rfc7946).
    """

    ic_contours_meter: Optional[int] = None
    """
    For a geofence based on isochrone contour determined using a specific driving
    distance, this property returns the duration value, in meters.

    The value would be the same as that provided for the `contours_meter` parameter
    at the time of creating or updating the geofence.
    """

    ic_contours_minute: Optional[int] = None
    """
    For a geofence based on isochrone contour determined using a specific driving
    duration, this property returns the duration value, in minutes. The value would
    be the same as the value provided for the `contours_minute` parameter at the
    time of creating or updating the geofence.
    """

    ic_coordinates: Optional[str] = None
    """
    For a geofence based on isochrone contour, this property returns the coordinates
    of the location, in [latitude,longitude] format, which was used as the starting
    point to identify the geofence boundary.

    The value would be the same as that provided for the `coordinates` parameter at
    the time of creating or updating the geofence.
    """

    ic_denoise: Optional[float] = None
    """
    For a geofence based on isochrone contour, this property returns the denoise
    value which would be the same as that provided for the `denoise` parameter at
    the time of creating or updating the geofence.
    """

    ic_departure_time: Optional[int] = None
    """
    For a geofence based on isochrone contour, this property returns the departure
    time, as a UNIX epoch timestamp in seconds, which was used to determine the
    geofence boundary after taking into account the traffic conditions at the time.

    The value would be the same as that provided for the `departure_time` parameter
    at the time of creating or updating the geofence.
    """

    ic_mode: Optional[float] = None
    """
    For a geofence based on isochrone contour, this property returns the driving
    mode used to determine the geofence boundary.

    The value would be the same as that provided for the `mode` parameter at the
    time of creating or updating the geofence.
    """

    meta_data: Optional[object] = None
    """Metadata of the geofence added at the time of creating or updating it."""

    name: Optional[str] = None
    """Name of the geofence added at the time of creating or updating it."""

    tags: Optional[List[str]] = None
    """
    An array of strings representing the `tags` associated with the geofence added
    at the time of creating or updating it.
    """

    type: Optional[Literal["`circle`", "`polygon`", "`isochrone`"]] = None
    """Type of the geofence."""

    updated_at: Optional[int] = None
    """
    Time at which the geofence was last updated, expressed as a UNIX timestamp in
    seconds.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel
from .route_steps_response import RouteStepsResponse

__all__ = ["StepUpdateResponse"]


class StepUpdateResponse(BaseModel):
    data: Optional[RouteStepsResponse] = None

    message: Optional[str] = None
    """Returns the error message in case of a failed request.

    If the request is successful, this field is not present in the response.
    """

    status: Optional[int] = None
    """Returns the status code of the response."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = [
    "PostalcodeRetrieveCoordinatesResponse",
    "Places",
    "PlacesBoundary",
    "PlacesBoundaryGeometry",
    "PlacesBoundaryMultipolygon",
    "PlacesBoundaryMultipolygonPolygon",
    "PlacesBoundaryMultipolygonPolygonPoint",
    "PlacesGeopoint",
]


class PlacesBoundaryGeometry(BaseModel):
    coordinates: Optional[List[List[List[float]]]] = None
    """
    An array of coordinates in the [longitude, latitude] format, representing the
    coordinates points which lie on the boundary of the postal code area.
    """

    type: Optional[str] = None
    """Type of the geoJSON geometry."""


class PlacesBoundaryMultipolygonPolygonPoint(BaseModel):
    lat: Optional[float] = None
    """Latitude of the coordinate."""

    lng: Optional[float] = None
    """Longitude of the coordinate."""


class PlacesBoundaryMultipolygonPolygon(BaseModel):
    points: Optional[List[PlacesBoundaryMultipolygonPolygonPoint]] = None
    """Represents an array of geographic coordinates that define a `polygon` boundary."""


class PlacesBoundaryMultipolygon(BaseModel):
    polygon: Optional[List[PlacesBoundaryMultipolygonPolygon]] = None
    """
    An object containing the details of a single polygon that is a part of the
    postal code area. In case the postal code area contains other polygon(s), the
    details of such polygon(s) would be returned through an array of `points`
    object.
    """


class PlacesBoundary(BaseModel):
    geometry: Optional[PlacesBoundaryGeometry] = None
    """An object with geoJSON details of the boundary.

    This object is returned when the `format` field is set to `geojson` in the input
    request, otherwise it is not present in the response. The contents of this
    object follow the
    [geoJSON standard](https://datatracker.ietf.org/doc/html/rfc7946).
    """

    multipolygon: Optional[List[PlacesBoundaryMultipolygon]] = None
    """
    An array of objects containing information about all the polygons forming the
    postal code area. In case, the postal code area is formed by multiple polygons
    not containing each other, a matching count of `polygon` objects will be
    returned.

    Please note that this object is returned only when `format` field is not
    specified in the input, otherwise it is not present in the response.
    """

    properties: Optional[str] = None
    """Property associated with the geoJSON shape."""

    type: Optional[str] = None
    """Type of the geoJSON object.

    This parameter is returned when the `format` field is set to `geojson` in the
    input request, otherwise it is not present in the response. The contents of this
    object follow the
    [geoJSON standard](https://datatracker.ietf.org/doc/html/rfc7946).
    """


class PlacesGeopoint(BaseModel):
    lat: Optional[float] = None
    """Latitude of the location."""

    lng: Optional[float] = None
    """Longitude of the location."""


class Places(BaseModel):
    address: Optional[str] = None
    """Returns the address of the `postalcode` returned."""

    boundary: Optional[PlacesBoundary] = None
    """An object containing the boundary details of the postal code area.

    This object will not be returned in case the boundary information of the postal
    code provided is not available (only for selected countries).

    Please note the contents of this object will change based on the `format` field
    in the input. When the `format` field is not present in the input this object
    would contain `multipolygon` - `polygon` - `points` objects depending on the
    boundary of the given postal code. When the `format` field is present in the
    input, then the contents of this object would match the
    [geojson format and standard](https://datatracker.ietf.org/doc/html/rfc7946).
    """

    country: Optional[str] = None
    """
    Name of the country containing the geographic coordinate point / postal code
    provided in the input request.
    """

    country_code: Optional[str] = FieldInfo(alias="countryCode", default=None)
    """
    Returns the [alpha-3 ISO code](https://www.iban.com/country-codes) of the
    country containing the `postalcode` returned.
    """

    distance: Optional[float] = None
    """
    This property is returned only when the API is requested to fetch the postal
    code containing the location coordinate provided in the `at` input parameter.
    `distance` denotes the straight line distance, in meters, from the requested
    location coordinate to the postal code centroid.
    """

    district: Optional[str] = None
    """
    Name of the district or region containing the geographic coordinate point /
    postal code provided in the input request.
    """

    geopoint: Optional[PlacesGeopoint] = None
    """
    Refers to the geographic coordinate denoting the center of the postal code in
    latitude, longitude format.
    """

    postalcode: Optional[str] = None
    """
    Returns the postal code associated with the requested geographic coordinate
    point or the postal code itself as provided in the input API request.
    """

    state: Optional[str] = None
    """
    Name of the state or province containing the geographic coordinate point /
    postal code provided in the input request.
    """

    subdistrict: Optional[str] = None
    """
    Name of the sub-district or sub-region containing the postal code or geographic
    coordinate point / postal code provided in the input request
    """


class PostalcodeRetrieveCoordinatesResponse(BaseModel):
    places: Optional[Places] = None
    """An object that contains details about the place that was provided in the input."""

    warning: Optional[List[str]] = None
    """Returns a message, in case the input provided triggers any warnings."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["PostResponse"]


class PostResponse(BaseModel):
    id: Optional[str] = None
    """
    A unique ID which can be used in the Optimization GET method to retrieve the
    result of optimization.
    """

    message: Optional[str] = None
    """Displays an acknowledgement message once the job is submitted."""

    status: Optional[str] = None
    """A string indicating the state of the response.

    On successful responses, the value will be `Ok`. Indicative error messages/codes
    are returned in case of errors. See the [API Error Codes](#api-error-codes)
    section below for more information.
    """

    warnings: Optional[List[str]] = None
    """Display the warnings for the given input parameters, values and constraints."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["RoutingResponse"]


class RoutingResponse(BaseModel):
    approaches: Optional[str] = None
    """
    Returns the configuration of approaches for each step, that is used when
    generating the route to help the driver with turn-by-turn navigation.
    """

    avoid: Optional[str] = None
    """
    Returns the objects and maneuvers that will be avoided in the route that is
    built when driver starts the in-app turn-by-turn navigation.
    """

    hazmat_type: Optional[str] = None
    """Returns the hazardous cargo type that the truck is carrying.

    The hazardous cargo type is used to determine the compliant routes that the
    driver can take while navigating the dispatched route.
    """

    mode: Optional[str] = None
    """
    Returns the driving mode that is used to build the route when driver starts the
    in-app turn-by-turn navigation.
    """

    truck_axle_load: Optional[str] = None
    """
    Returns the total load per axle of the truck, in tonnes, used to determine
    compliant routes that the driver can take when he starts navigating the
    dispatched route.
    """

    truck_size: Optional[str] = None
    """
    Returns the truck dimensions, in centimeters, used to determine compliant routes
    that the driver can take when he starts navigating the dispatched route.
    """

    truck_weight: Optional[str] = None
    """
    Returns the truck weight that will determine compliant routes that can be used
    by the driver during navigation.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Monitor", "GeofenceConfig", "IdleConfig", "MatchFilter", "SpeedingConfig"]


class GeofenceConfig(BaseModel):
    geofence_ids: Optional[List[str]] = None
    """An array of geofence IDs that are linked to the `monitor`.

    Geofences are geographic boundaries that can be used to trigger events based on
    an asset's location.
    """


class IdleConfig(BaseModel):
    distance_tolerance: Optional[float] = None
    """
    This parameter returns the distance threshold that was used to determine if the
    asset was idle or not. The value returned for this parameter is the same as that
    provided while creating or updating a `idle` type `monitor`.
    """

    time_tolerance: Optional[int] = None
    """
    This parameter returns the time duration for which the `monitor` tracks the
    distance covered by an asset before triggering an idle event. The value returned
    for this parameter is the same as that provided while creating or updating a
    `idle` type `monitor`.
    """


class MatchFilter(BaseModel):
    include_all_of_attributes: Optional[object] = None
    """
    A string type dictionary object to specify the `attributes` which will be used
    to identify the asset(s) on which the `monitor` would be applied. Please note
    that using this parameter overwrites the existing `attributes` of the monitor.

    If the `attributes` added to a `monitor` do not match fully with the
    `attributes` added to any `asset`, the `monitor` will be ineffective.

    Please note that the maximum number of `key`:`value` pairs that
    'include_all_of_attributes' can take is 100. Also, the overall size of the
    `match_filter` object should not exceed 65kb.
    """

    include_any_of_attributes: Optional[object] = None
    """A string dictionary object to specify the `attributes`, separated by a `,`.

    Only the `assets` with any one of the `attributes` added to this parameter will
    be linked to this `monitor`. Once an `asset` and a `monitor` are linked, the
    `monitor` will be able to create events for the `asset` when an activity
    specified in `type` is detected.

    If no input is provided for this object or if the `attributes` added here do not
    match at least one of the `attributes` added to any `asset`, the `monitor` will
    be ineffective.

    Please note that the maximum number of `key`:`value` pairs that
    `include_any_of_attributes` can take is 100. Also, the overall size of
    `match_filter` object should not exceed 65kb.
    """


class SpeedingConfig(BaseModel):
    customer_speed_limit: Optional[int] = None
    """
    This property returns the actual speed limit that the `monitor` uses as a
    threshold for generating a speed limit event. The value returned for this
    parameter is the same as that provided while creating or updating a `speeding`
    type `monitor`.
    """

    time_tolerance: Optional[int] = None
    """
    This property returns the time duration value, in milliseconds, for which the
    `monitor` will track the speed of the asset. An event is triggered if the speed
    remains higher than the specified limit for a duration more than the tolerance
    value.

    The value returned for this parameter is the same as that provided while
    creating or updating a `speeding` type `monitor`.
    """

    use_admin_speed_limit: Optional[bool] = None
    """
    A boolean value denoting if the administrative speed limit of the road was used
    as speed limit threshold for triggering events. The value returned for this
    parameter is the same as that provided while creating or updating a `speeding`
    type `monitor`.
    """


class Monitor(BaseModel):
    id: Optional[str] = None
    """Unique ID of the `monitor`.

    This is the same ID that was generated at the time of creating the `monitor`.
    """

    created_at: Optional[int] = None
    """
    A UNIX epoch timestamp in seconds representing the time at which the `monitor`
    was created.
    """

    description: Optional[str] = None
    """Description of the `monitor`.

    The value would be the same as that provided for the `description` parameter at
    the time of creating or updating the `monitor`.
    """

    geofence_config: Optional[GeofenceConfig] = None
    """
    An object returning the details of the geofence that are associated with the
    `monitor` for an `enter`, `exit` or `enter_and_exit` type of monitor.
    """

    geofences: Optional[List[str]] = None
    """Geofence IDs that are linked to the `monitor`.

    These IDs were associated with the `monitor` at the time of creating or updating
    it.

    The `monitor` uses the geofences mentioned here to create events of `type`
    nature for the eligible asset(s).
    """

    idle_config: Optional[IdleConfig] = None
    """
    An object returning the details of the idle activity constraints for a `idle`
    type of `monitor`.
    """

    match_filter: Optional[MatchFilter] = None
    """Use this object to update the `attributes` of the `monitor`."""

    meta_data: Optional[object] = None
    """Any valid json object data.

    Can be used to save customized data. Max size is 65kb.
    """

    name: Optional[str] = None
    """Name of the `monitor`.

    The value would be the same as that provided for the `name` parameter at the
    time of creating or updating the `monitor`.
    """

    speeding_config: Optional[SpeedingConfig] = None
    """
    An object returning the details of the over-speeding constraints for a
    `speeding` type of `monitor`.
    """

    tags: Optional[List[str]] = None
    """Tags of the `monitor`.

    The values would be the same as that provided for the `tags` parameter at the
    time of creating or updating the `monitor`.
    """

    type: Optional[Literal["`enter`", "`exit`", "`enter_and_exit`", "`speeding`", "`idle`"]] = None
    """Type of the `monitor`.

    It represents the type of `asset` activity that the `monitor` is configured to
    detect.
    """

    updated_at: Optional[int] = None
    """
    A UNIX epoch timestamp in seconds representing the time at which the `monitor`
    was last updated.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.skynet.search import polygon_get_params, polygon_create_params
from ....types.skynet.search_response import SearchResponse

__all__ = ["PolygonResource", "AsyncPolygonResource"]


class PolygonResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PolygonResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return PolygonResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PolygonResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return PolygonResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        key: str,
        polygon: polygon_create_params.Polygon,
        filter: str | NotGiven = NOT_GIVEN,
        match_filter: polygon_create_params.MatchFilter | NotGiven = NOT_GIVEN,
        max_search_limit: bool | NotGiven = NOT_GIVEN,
        pn: int | NotGiven = NOT_GIVEN,
        ps: int | NotGiven = NOT_GIVEN,
        sort: polygon_create_params.Sort | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SearchResponse:
        """
        Polygon Search

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          polygon:
              An object to collect geoJSON details of a custom polygon. Please ensure that:

              - the `polygon` provided is enclosed. This can be achieved by making the last
                location coordinate in the list equal to the first location coordinate of the
                list.

              - the 'polygon' provided does not contain multiple rings.

              The contents of this object follow the
              [geoJSON standard](https://datatracker.ietf.org/doc/html/rfc7946).

              Please note that the maximum area of the search polygon allowed is 3000
              km<sup>2</sup>.

          filter: **`tags` parameter will be deprecated soon! Please use the
              `include_any_of_attributes` or `include_all_of_attributes` parameters to match
              assets based on their labels or markers.**

              Use this parameter to filter the assets found inside the specified area by their
              `tag`. Multiple `tag` can be separated using comma (`,`).

              Please note the tags are case sensitive.

          match_filter: An object to define the `attributes` which will be used to filter the assets
              found within the `polygon`.

          max_search_limit: if ture, can get 16x bigger limitation in search.

          pn: Denotes page number. Use this along with the `ps` parameter to implement
              pagination for your searched results. This parameter does not have a maximum
              limit but would return an empty response in case a higher value is provided when
              the result-set itself is smaller.

          ps: Denotes number of search results per page. Use this along with the `pn`
              parameter to implement pagination for your searched results. Please note that
              `ps` has a default value of 20 and accepts integers only in the range of [1,
              100].

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/skynet/search/polygon",
            body=maybe_transform(
                {
                    "polygon": polygon,
                    "filter": filter,
                    "match_filter": match_filter,
                    "max_search_limit": max_search_limit,
                    "pn": pn,
                    "ps": ps,
                    "sort": sort,
                },
                polygon_create_params.PolygonCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, polygon_create_params.PolygonCreateParams),
            ),
            cast_to=SearchResponse,
        )

    def get(
        self,
        *,
        key: str,
        polygon: str,
        filter: str | NotGiven = NOT_GIVEN,
        include_all_of_attributes: str | NotGiven = NOT_GIVEN,
        include_any_of_attributes: str | NotGiven = NOT_GIVEN,
        max_search_limit: bool | NotGiven = NOT_GIVEN,
        pn: int | NotGiven = NOT_GIVEN,
        ps: int | NotGiven = NOT_GIVEN,
        sort_by: Literal["`distance`", "`duration`", "`straight_distance`"] | NotGiven = NOT_GIVEN,
        sort_destination: str | NotGiven = NOT_GIVEN,
        sort_driving_mode: Literal["`car`", "`truck`"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SearchResponse:
        """
        Polygon Search

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          polygon: Define a custom polygon enclosing the area to be searched. It should be a pipe
              (`|`) delimited list of location coordinates.

              Please ensure that the `polygon` provided is enclosed. This can be achieved by
              making the last location coordinate in the list equal to the first location
              coordinate of the list.

              Please note that the maximum area of the search polygon allowed is 3000
              km<sup>2</sup>.

          filter: **`tags` parameter will be deprecated soon! Please use the
              `include_any_of_attributes` or `include_all_of_attributes` parameters to match
              assets based on their labels or markers.**

              Use this parameter to filter the assets found inside the specified area by their
              `tags`. Multiple `tags` can be separated using commas (`,`).

              Please note the tags are case sensitive.

          include_all_of_attributes: Use this parameter to filter the assets found inside the specified area by their
              `attributes`. Only the assets having all the `attributes` that are added to this
              parameter, will be returned in the search results. Multiple `attributes` can be
              separated using pipes (`|`).

              Please note the attributes are case sensitive. Also, this parameter can not be
              used in conjunction with `include_any_of_attributes` parameter.

          include_any_of_attributes: Use this parameter to filter the assets found inside the specified area by their
              `attributes`. Assets having at least one of the `attributes` added to this
              parameter, will be returned in the search results. Multiple `attributes` can be
              separated using pipes (`|`).

              Please note the attributes are case sensitive. Also, this parameter can not be
              used in conjunction with `include_all_of_attributes` parameter.

          max_search_limit: When true, the maximum limit is 20Km for around search API and 48000 Km2 for
              other search methods.

          pn: Denotes page number. Use this along with the `ps` parameter to implement
              pagination for your searched results. This parameter does not have a maximum
              limit but would return an empty response in case a higher value is provided when
              the result-set itself is smaller.

          ps: Denotes number of search results per page. Use this along with the `pn`
              parameter to implement pagination for your searched results.

          sort_by: Specify the metric to sort the assets returned in the search result. The valid
              values are:

              - **distance** : Sorts the assets by driving distance to the given
                `sort_destination` .
              - **duration** : Sorts the assets by travel time to the given `sort_destination`
                .
              - **straight_distance** : Sort the assets by straight-line distance to the given
                `sort-destination` .

          sort_destination: Specifies the location coordinates of the point which acts as destination for
              sorting the assets in the search results. The service will sort each asset based
              on the driving distance or travel time to this destination, from its current
              location. Use the `sort_by` parameter to configure the metric that should be
              used for sorting the assets. Please note that `sort_destination` is required
              when `sort_by` is provided.

          sort_driving_mode: Specifies the driving mode to be used for determining travel duration or driving
              distance for sorting the assets in search result.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/skynet/search/polygon",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "polygon": polygon,
                        "filter": filter,
                        "include_all_of_attributes": include_all_of_attributes,
                        "include_any_of_attributes": include_any_of_attributes,
                        "max_search_limit": max_search_limit,
                        "pn": pn,
                        "ps": ps,
                        "sort_by": sort_by,
                        "sort_destination": sort_destination,
                        "sort_driving_mode": sort_driving_mode,
                    },
                    polygon_get_params.PolygonGetParams,
                ),
            ),
            cast_to=SearchResponse,
        )


class AsyncPolygonResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPolygonResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPolygonResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPolygonResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncPolygonResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        key: str,
        polygon: polygon_create_params.Polygon,
        filter: str | NotGiven = NOT_GIVEN,
        match_filter: polygon_create_params.MatchFilter | NotGiven = NOT_GIVEN,
        max_search_limit: bool | NotGiven = NOT_GIVEN,
        pn: int | NotGiven = NOT_GIVEN,
        ps: int | NotGiven = NOT_GIVEN,
        sort: polygon_create_params.Sort | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SearchResponse:
        """
        Polygon Search

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          polygon:
              An object to collect geoJSON details of a custom polygon. Please ensure that:

              - the `polygon` provided is enclosed. This can be achieved by making the last
                location coordinate in the list equal to the first location coordinate of the
                list.

              - the 'polygon' provided does not contain multiple rings.

              The contents of this object follow the
              [geoJSON standard](https://datatracker.ietf.org/doc/html/rfc7946).

              Please note that the maximum area of the search polygon allowed is 3000
              km<sup>2</sup>.

          filter: **`tags` parameter will be deprecated soon! Please use the
              `include_any_of_attributes` or `include_all_of_attributes` parameters to match
              assets based on their labels or markers.**

              Use this parameter to filter the assets found inside the specified area by their
              `tag`. Multiple `tag` can be separated using comma (`,`).

              Please note the tags are case sensitive.

          match_filter: An object to define the `attributes` which will be used to filter the assets
              found within the `polygon`.

          max_search_limit: if ture, can get 16x bigger limitation in search.

          pn: Denotes page number. Use this along with the `ps` parameter to implement
              pagination for your searched results. This parameter does not have a maximum
              limit but would return an empty response in case a higher value is provided when
              the result-set itself is smaller.

          ps: Denotes number of search results per page. Use this along with the `pn`
              parameter to implement pagination for your searched results. Please note that
              `ps` has a default value of 20 and accepts integers only in the range of [1,
              100].

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/skynet/search/polygon",
            body=await async_maybe_transform(
                {
                    "polygon": polygon,
                    "filter": filter,
                    "match_filter": match_filter,
                    "max_search_limit": max_search_limit,
                    "pn": pn,
                    "ps": ps,
                    "sort": sort,
                },
                polygon_create_params.PolygonCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, polygon_create_params.PolygonCreateParams),
            ),
            cast_to=SearchResponse,
        )

    async def get(
        self,
        *,
        key: str,
        polygon: str,
        filter: str | NotGiven = NOT_GIVEN,
        include_all_of_attributes: str | NotGiven = NOT_GIVEN,
        include_any_of_attributes: str | NotGiven = NOT_GIVEN,
        max_search_limit: bool | NotGiven = NOT_GIVEN,
        pn: int | NotGiven = NOT_GIVEN,
        ps: int | NotGiven = NOT_GIVEN,
        sort_by: Literal["`distance`", "`duration`", "`straight_distance`"] | NotGiven = NOT_GIVEN,
        sort_destination: str | NotGiven = NOT_GIVEN,
        sort_driving_mode: Literal["`car`", "`truck`"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SearchResponse:
        """
        Polygon Search

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          polygon: Define a custom polygon enclosing the area to be searched. It should be a pipe
              (`|`) delimited list of location coordinates.

              Please ensure that the `polygon` provided is enclosed. This can be achieved by
              making the last location coordinate in the list equal to the first location
              coordinate of the list.

              Please note that the maximum area of the search polygon allowed is 3000
              km<sup>2</sup>.

          filter: **`tags` parameter will be deprecated soon! Please use the
              `include_any_of_attributes` or `include_all_of_attributes` parameters to match
              assets based on their labels or markers.**

              Use this parameter to filter the assets found inside the specified area by their
              `tags`. Multiple `tags` can be separated using commas (`,`).

              Please note the tags are case sensitive.

          include_all_of_attributes: Use this parameter to filter the assets found inside the specified area by their
              `attributes`. Only the assets having all the `attributes` that are added to this
              parameter, will be returned in the search results. Multiple `attributes` can be
              separated using pipes (`|`).

              Please note the attributes are case sensitive. Also, this parameter can not be
              used in conjunction with `include_any_of_attributes` parameter.

          include_any_of_attributes: Use this parameter to filter the assets found inside the specified area by their
              `attributes`. Assets having at least one of the `attributes` added to this
              parameter, will be returned in the search results. Multiple `attributes` can be
              separated using pipes (`|`).

              Please note the attributes are case sensitive. Also, this parameter can not be
              used in conjunction with `include_all_of_attributes` parameter.

          max_search_limit: When true, the maximum limit is 20Km for around search API and 48000 Km2 for
              other search methods.

          pn: Denotes page number. Use this along with the `ps` parameter to implement
              pagination for your searched results. This parameter does not have a maximum
              limit but would return an empty response in case a higher value is provided when
              the result-set itself is smaller.

          ps: Denotes number of search results per page. Use this along with the `pn`
              parameter to implement pagination for your searched results.

          sort_by: Specify the metric to sort the assets returned in the search result. The valid
              values are:

              - **distance** : Sorts the assets by driving distance to the given
                `sort_destination` .
              - **duration** : Sorts the assets by travel time to the given `sort_destination`
                .
              - **straight_distance** : Sort the assets by straight-line distance to the given
                `sort-destination` .

          sort_destination: Specifies the location coordinates of the point which acts as destination for
              sorting the assets in the search results. The service will sort each asset based
              on the driving distance or travel time to this destination, from its current
              location. Use the `sort_by` parameter to configure the metric that should be
              used for sorting the assets. Please note that `sort_destination` is required
              when `sort_by` is provided.

          sort_driving_mode: Specifies the driving mode to be used for determining travel duration or driving
              distance for sorting the assets in search result.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/skynet/search/polygon",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "polygon": polygon,
                        "filter": filter,
                        "include_all_of_attributes": include_all_of_attributes,
                        "include_any_of_attributes": include_any_of_attributes,
                        "max_search_limit": max_search_limit,
                        "pn": pn,
                        "ps": ps,
                        "sort_by": sort_by,
                        "sort_destination": sort_destination,
                        "sort_driving_mode": sort_driving_mode,
                    },
                    polygon_get_params.PolygonGetParams,
                ),
            ),
            cast_to=SearchResponse,
        )


class PolygonResourceWithRawResponse:
    def __init__(self, polygon: PolygonResource) -> None:
        self._polygon = polygon

        self.create = to_raw_response_wrapper(
            polygon.create,
        )
        self.get = to_raw_response_wrapper(
            polygon.get,
        )


class AsyncPolygonResourceWithRawResponse:
    def __init__(self, polygon: AsyncPolygonResource) -> None:
        self._polygon = polygon

        self.create = async_to_raw_response_wrapper(
            polygon.create,
        )
        self.get = async_to_raw_response_wrapper(
            polygon.get,
        )


class PolygonResourceWithStreamingResponse:
    def __init__(self, polygon: PolygonResource) -> None:
        self._polygon = polygon

        self.create = to_streamed_response_wrapper(
            polygon.create,
        )
        self.get = to_streamed_response_wrapper(
            polygon.get,
        )


class AsyncPolygonResourceWithStreamingResponse:
    def __init__(self, polygon: AsyncPolygonResource) -> None:
        self._polygon = polygon

        self.create = async_to_streamed_response_wrapper(
            polygon.create,
        )
        self.get = async_to_streamed_response_wrapper(
            polygon.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["PolygonCreateParams", "Polygon", "MatchFilter", "Sort", "SortSortDestination"]


class PolygonCreateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    polygon: Required[Polygon]
    """An object to collect geoJSON details of a custom polygon. Please ensure that:

    - the `polygon` provided is enclosed. This can be achieved by making the last
      location coordinate in the list equal to the first location coordinate of the
      list.

    - the 'polygon' provided does not contain multiple rings.

    The contents of this object follow the
    [geoJSON standard](https://datatracker.ietf.org/doc/html/rfc7946).

    Please note that the maximum area of the search polygon allowed is 3000
    km<sup>2</sup>.
    """

    filter: str
    """
    **`tags` parameter will be deprecated soon! Please use the
    `include_any_of_attributes` or `include_all_of_attributes` parameters to match
    assets based on their labels or markers.**

    Use this parameter to filter the assets found inside the specified area by their
    `tag`. Multiple `tag` can be separated using comma (`,`).

    Please note the tags are case sensitive.
    """

    match_filter: MatchFilter
    """
    An object to define the `attributes` which will be used to filter the assets
    found within the `polygon`.
    """

    max_search_limit: bool
    """if ture, can get 16x bigger limitation in search."""

    pn: int
    """Denotes page number.

    Use this along with the `ps` parameter to implement pagination for your searched
    results. This parameter does not have a maximum limit but would return an empty
    response in case a higher value is provided when the result-set itself is
    smaller.
    """

    ps: int
    """Denotes number of search results per page.

    Use this along with the `pn` parameter to implement pagination for your searched
    results. Please note that `ps` has a default value of 20 and accepts integers
    only in the range of [1, 100].
    """

    sort: Sort


class Polygon(TypedDict, total=False):
    coordinates: Required[Iterable[float]]
    """
    An array of coordinates in the [longitude, latitude] format, representing the
    polygon boundary.
    """

    type: Required[str]
    """Type of the geoJSON geometry. Should always be `polygon`."""


class MatchFilter(TypedDict, total=False):
    include_all_of_attributes: str
    """
    Use this parameter to filter the assets found inside the specified area by their
    `attributes`. Only the assets having all the `attributes` that are added to this
    parameter, will be returned in the search results. Multiple `attributes` can be
    separated using commas (`,`).

    Please note the attributes are case sensitive. Also, this parameter can not be
    used in conjunction with `include_any_of_attributes` parameter.
    """

    include_any_of_attributes: str
    """
    Use this parameter to filter the assets found inside the specified area by their
    `attributes`. Assets having at least one of the `attributes` added to this
    parameter, will be returned in the search results. Multiple `attributes` can be
    separated using commas (`,`).

    Please note the attributes are case sensitive. Also, this parameter can not be
    used in conjunction with `include_all_of_attributes` parameter.
    """


class SortSortDestination(TypedDict, total=False):
    lat: Required[float]
    """Latitude of the destination location"""

    lon: Required[float]
    """Longitude of the destination location"""


class Sort(TypedDict, total=False):
    sort_by: Literal["`distance`", "`duration`", "`straight_distance`"]
    """Specify the metric to sort the assets returned in the search result.

    The valid values are:

    - **distance** : Sorts the assets by driving distance to the given
      `sort_destination` .
    - **duration** : Sorts the assets by travel time to the given `sort_destination`
      .
    - **straight_distance** : Sort the assets by straight-line distance to the given
      `sort-destination` .
    """

    sort_destination: SortSortDestination
    """
    Specifies the location coordinates of the point which acts as destination for
    sorting the assets in the search results. The service will sort each asset based
    on the driving distance or travel time to this destination, from its current
    location. Use the `sort_by` parameter to configure the metric that should be
    used for sorting the assets. Please note that `sort_destination` is required
    when `sort_by` is provided.
    """

    sort_driving_mode: Literal["`car`", "`truck`"]
    """
    Specifies the driving mode to be used for determining travel duration or driving
    distance for sorting the assets in search result.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types.skynet import (
    SimpleResp,
    TripStartResponse,
    TripRetrieveResponse,
    TripGetSummaryResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTrip:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: NextbillionSDK) -> None:
        trip = client.skynet.trip.retrieve(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(TripRetrieveResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_with_all_params(self, client: NextbillionSDK) -> None:
        trip = client.skynet.trip.retrieve(
            id="id",
            key="key=API_KEY",
            cluster="america",
        )
        assert_matches_type(TripRetrieveResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: NextbillionSDK) -> None:
        response = client.skynet.trip.with_raw_response.retrieve(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        trip = response.parse()
        assert_matches_type(TripRetrieveResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: NextbillionSDK) -> None:
        with client.skynet.trip.with_streaming_response.retrieve(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            trip = response.parse()
            assert_matches_type(TripRetrieveResponse, trip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.skynet.trip.with_raw_response.retrieve(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: NextbillionSDK) -> None:
        trip = client.skynet.trip.update(
            id="id",
            key="key=API_KEY",
            asset_id="asset_id",
        )
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: NextbillionSDK) -> None:
        trip = client.skynet.trip.update(
            id="id",
            key="key=API_KEY",
            asset_id="asset_id",
            cluster="america",
            attributes='{\n  "shift_timing": "0800-1700",\n  "driver_name": "John"\n}',
            description="description",
            meta_data='"meta_data":["Scheduled Trip", "Custom Deliveries"]',
            name='"name": "Employee Pickup"',
            stops=[
                {
                    "geofence_id": "geofence_id",
                    "meta_data": '"meta_data":["Staff Entry Point", "Biometric checkpoint"]',
                    "name": '"name":"Head Office"',
                }
            ],
        )
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: NextbillionSDK) -> None:
        response = client.skynet.trip.with_raw_response.update(
            id="id",
            key="key=API_KEY",
            asset_id="asset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        trip = response.parse()
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: NextbillionSDK) -> None:
        with client.skynet.trip.with_streaming_response.update(
            id="id",
            key="key=API_KEY",
            asset_id="asset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            trip = response.parse()
            assert_matches_type(SimpleResp, trip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.skynet.trip.with_raw_response.update(
                id="",
                key="key=API_KEY",
                asset_id="asset_id",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: NextbillionSDK) -> None:
        trip = client.skynet.trip.delete(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_delete_with_all_params(self, client: NextbillionSDK) -> None:
        trip = client.skynet.trip.delete(
            id="id",
            key="key=API_KEY",
            cluster="america",
        )
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: NextbillionSDK) -> None:
        response = client.skynet.trip.with_raw_response.delete(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        trip = response.parse()
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: NextbillionSDK) -> None:
        with client.skynet.trip.with_streaming_response.delete(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            trip = response.parse()
            assert_matches_type(SimpleResp, trip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_delete(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.skynet.trip.with_raw_response.delete(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_end(self, client: NextbillionSDK) -> None:
        trip = client.skynet.trip.end(
            key="key=API_KEY",
            id="id",
        )
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_end_with_all_params(self, client: NextbillionSDK) -> None:
        trip = client.skynet.trip.end(
            key="key=API_KEY",
            id="id",
            cluster="america",
        )
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_end(self, client: NextbillionSDK) -> None:
        response = client.skynet.trip.with_raw_response.end(
            key="key=API_KEY",
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        trip = response.parse()
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_end(self, client: NextbillionSDK) -> None:
        with client.skynet.trip.with_streaming_response.end(
            key="key=API_KEY",
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            trip = response.parse()
            assert_matches_type(SimpleResp, trip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_get_summary(self, client: NextbillionSDK) -> None:
        trip = client.skynet.trip.get_summary(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(TripGetSummaryResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get_summary_with_all_params(self, client: NextbillionSDK) -> None:
        trip = client.skynet.trip.get_summary(
            id="id",
            key="key=API_KEY",
            cluster="america",
        )
        assert_matches_type(TripGetSummaryResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get_summary(self, client: NextbillionSDK) -> None:
        response = client.skynet.trip.with_raw_response.get_summary(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        trip = response.parse()
        assert_matches_type(TripGetSummaryResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get_summary(self, client: NextbillionSDK) -> None:
        with client.skynet.trip.with_streaming_response.get_summary(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            trip = response.parse()
            assert_matches_type(TripGetSummaryResponse, trip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_get_summary(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.skynet.trip.with_raw_response.get_summary(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_start(self, client: NextbillionSDK) -> None:
        trip = client.skynet.trip.start(
            key="key=API_KEY",
            asset_id="asset_id",
        )
        assert_matches_type(TripStartResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_start_with_all_params(self, client: NextbillionSDK) -> None:
        trip = client.skynet.trip.start(
            key="key=API_KEY",
            asset_id="asset_id",
            cluster="america",
            attributes='{\n  "shift_timing": "0800-1700",\n  "driver_name": "John"\n}',
            custom_id="custom_id",
            description="description",
            meta_data='"meta_data":["Scheduled Trip", "Custom Deliveries"]',
            name='"name": "Employee Pickup"',
            stops=[
                {
                    "geofence_id": "geofence_id",
                    "meta_data": '"meta_data":["Staff Entry Point", "Biometric checkpoint"]',
                    "name": '"name":"Head Office"',
                }
            ],
        )
        assert_matches_type(TripStartResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_start(self, client: NextbillionSDK) -> None:
        response = client.skynet.trip.with_raw_response.start(
            key="key=API_KEY",
            asset_id="asset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        trip = response.parse()
        assert_matches_type(TripStartResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_start(self, client: NextbillionSDK) -> None:
        with client.skynet.trip.with_streaming_response.start(
            key="key=API_KEY",
            asset_id="asset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            trip = response.parse()
            assert_matches_type(TripStartResponse, trip, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncTrip:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        trip = await async_client.skynet.trip.retrieve(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(TripRetrieveResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        trip = await async_client.skynet.trip.retrieve(
            id="id",
            key="key=API_KEY",
            cluster="america",
        )
        assert_matches_type(TripRetrieveResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.trip.with_raw_response.retrieve(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        trip = await response.parse()
        assert_matches_type(TripRetrieveResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.trip.with_streaming_response.retrieve(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            trip = await response.parse()
            assert_matches_type(TripRetrieveResponse, trip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.skynet.trip.with_raw_response.retrieve(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncNextbillionSDK) -> None:
        trip = await async_client.skynet.trip.update(
            id="id",
            key="key=API_KEY",
            asset_id="asset_id",
        )
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        trip = await async_client.skynet.trip.update(
            id="id",
            key="key=API_KEY",
            asset_id="asset_id",
            cluster="america",
            attributes='{\n  "shift_timing": "0800-1700",\n  "driver_name": "John"\n}',
            description="description",
            meta_data='"meta_data":["Scheduled Trip", "Custom Deliveries"]',
            name='"name": "Employee Pickup"',
            stops=[
                {
                    "geofence_id": "geofence_id",
                    "meta_data": '"meta_data":["Staff Entry Point", "Biometric checkpoint"]',
                    "name": '"name":"Head Office"',
                }
            ],
        )
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.trip.with_raw_response.update(
            id="id",
            key="key=API_KEY",
            asset_id="asset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        trip = await response.parse()
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.trip.with_streaming_response.update(
            id="id",
            key="key=API_KEY",
            asset_id="asset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            trip = await response.parse()
            assert_matches_type(SimpleResp, trip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.skynet.trip.with_raw_response.update(
                id="",
                key="key=API_KEY",
                asset_id="asset_id",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncNextbillionSDK) -> None:
        trip = await async_client.skynet.trip.delete(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        trip = await async_client.skynet.trip.delete(
            id="id",
            key="key=API_KEY",
            cluster="america",
        )
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.trip.with_raw_response.delete(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        trip = await response.parse()
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.trip.with_streaming_response.delete(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            trip = await response.parse()
            assert_matches_type(SimpleResp, trip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.skynet.trip.with_raw_response.delete(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_end(self, async_client: AsyncNextbillionSDK) -> None:
        trip = await async_client.skynet.trip.end(
            key="key=API_KEY",
            id="id",
        )
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_end_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        trip = await async_client.skynet.trip.end(
            key="key=API_KEY",
            id="id",
            cluster="america",
        )
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_end(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.trip.with_raw_response.end(
            key="key=API_KEY",
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        trip = await response.parse()
        assert_matches_type(SimpleResp, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_end(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.trip.with_streaming_response.end(
            key="key=API_KEY",
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            trip = await response.parse()
            assert_matches_type(SimpleResp, trip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_summary(self, async_client: AsyncNextbillionSDK) -> None:
        trip = await async_client.skynet.trip.get_summary(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(TripGetSummaryResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_summary_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        trip = await async_client.skynet.trip.get_summary(
            id="id",
            key="key=API_KEY",
            cluster="america",
        )
        assert_matches_type(TripGetSummaryResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get_summary(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.trip.with_raw_response.get_summary(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        trip = await response.parse()
        assert_matches_type(TripGetSummaryResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get_summary(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.trip.with_streaming_response.get_summary(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            trip = await response.parse()
            assert_matches_type(TripGetSummaryResponse, trip, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_get_summary(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.skynet.trip.with_raw_response.get_summary(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_start(self, async_client: AsyncNextbillionSDK) -> None:
        trip = await async_client.skynet.trip.start(
            key="key=API_KEY",
            asset_id="asset_id",
        )
        assert_matches_type(TripStartResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_start_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        trip = await async_client.skynet.trip.start(
            key="key=API_KEY",
            asset_id="asset_id",
            cluster="america",
            attributes='{\n  "shift_timing": "0800-1700",\n  "driver_name": "John"\n}',
            custom_id="custom_id",
            description="description",
            meta_data='"meta_data":["Scheduled Trip", "Custom Deliveries"]',
            name='"name": "Employee Pickup"',
            stops=[
                {
                    "geofence_id": "geofence_id",
                    "meta_data": '"meta_data":["Staff Entry Point", "Biometric checkpoint"]',
                    "name": '"name":"Head Office"',
                }
            ],
        )
        assert_matches_type(TripStartResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_start(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.trip.with_raw_response.start(
            key="key=API_KEY",
            asset_id="asset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        trip = await response.parse()
        assert_matches_type(TripStartResponse, trip, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_start(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.trip.with_streaming_response.start(
            key="key=API_KEY",
            asset_id="asset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            trip = await response.parse()
            assert_matches_type(TripStartResponse, trip, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types import RestrictionsItemListResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRestrictionsItems:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: NextbillionSDK) -> None:
        restrictions_item = client.restrictions_items.list(
            max_lat=0,
            max_lon=0,
            min_lat=0,
            min_lon=0,
        )
        assert_matches_type(RestrictionsItemListResponse, restrictions_item, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: NextbillionSDK) -> None:
        restrictions_item = client.restrictions_items.list(
            max_lat=0,
            max_lon=0,
            min_lat=0,
            min_lon=0,
            group_id=0,
            mode="0w",
            restriction_type="turn",
            source="source",
            state="enabled",
            status="active",
        )
        assert_matches_type(RestrictionsItemListResponse, restrictions_item, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: NextbillionSDK) -> None:
        response = client.restrictions_items.with_raw_response.list(
            max_lat=0,
            max_lon=0,
            min_lat=0,
            min_lon=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        restrictions_item = response.parse()
        assert_matches_type(RestrictionsItemListResponse, restrictions_item, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: NextbillionSDK) -> None:
        with client.restrictions_items.with_streaming_response.list(
            max_lat=0,
            max_lon=0,
            min_lat=0,
            min_lon=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            restrictions_item = response.parse()
            assert_matches_type(RestrictionsItemListResponse, restrictions_item, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncRestrictionsItems:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncNextbillionSDK) -> None:
        restrictions_item = await async_client.restrictions_items.list(
            max_lat=0,
            max_lon=0,
            min_lat=0,
            min_lon=0,
        )
        assert_matches_type(RestrictionsItemListResponse, restrictions_item, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        restrictions_item = await async_client.restrictions_items.list(
            max_lat=0,
            max_lon=0,
            min_lat=0,
            min_lon=0,
            group_id=0,
            mode="0w",
            restriction_type="turn",
            source="source",
            state="enabled",
            status="active",
        )
        assert_matches_type(RestrictionsItemListResponse, restrictions_item, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.restrictions_items.with_raw_response.list(
            max_lat=0,
            max_lon=0,
            min_lat=0,
            min_lon=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        restrictions_item = await response.parse()
        assert_matches_type(RestrictionsItemListResponse, restrictions_item, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.restrictions_items.with_streaming_response.list(
            max_lat=0,
            max_lon=0,
            min_lat=0,
            min_lon=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            restrictions_item = await response.parse()
            assert_matches_type(RestrictionsItemListResponse, restrictions_item, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types.skynet import (
    SimpleResp,
    MonitorListResponse,
    MonitorCreateResponse,
    MonitorRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMonitor:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: NextbillionSDK) -> None:
        monitor = client.skynet.monitor.create(
            key="key=API_KEY",
            tags=["string"],
            type="`enter`",
        )
        assert_matches_type(MonitorCreateResponse, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: NextbillionSDK) -> None:
        monitor = client.skynet.monitor.create(
            key="key=API_KEY",
            tags=["string"],
            type="`enter`",
            cluster="america",
            custom_id="custom_id",
            description="description",
            geofence_config={"geofence_ids": ["string"]},
            geofence_ids=["string"],
            idle_config={
                "distance_tolerance": 0,
                "time_tolerance": 0,
            },
            match_filter={
                "include_all_of_attributes": '{\n  "asset_type": "delivery",\n  "area": "Los Angeles downtown"\n}',
                "include_any_of_attributes": '{\n  "asset_type": "delivery",\n  "area": "Los Angeles downtown"\n}',
            },
            meta_data={},
            name="name",
            speeding_config={
                "customer_speed_limit": 0,
                "time_tolerance": 0,
                "use_admin_speed_limit": True,
            },
        )
        assert_matches_type(MonitorCreateResponse, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: NextbillionSDK) -> None:
        response = client.skynet.monitor.with_raw_response.create(
            key="key=API_KEY",
            tags=["string"],
            type="`enter`",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        monitor = response.parse()
        assert_matches_type(MonitorCreateResponse, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: NextbillionSDK) -> None:
        with client.skynet.monitor.with_streaming_response.create(
            key="key=API_KEY",
            tags=["string"],
            type="`enter`",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            monitor = response.parse()
            assert_matches_type(MonitorCreateResponse, monitor, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: NextbillionSDK) -> None:
        monitor = client.skynet.monitor.retrieve(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(MonitorRetrieveResponse, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: NextbillionSDK) -> None:
        response = client.skynet.monitor.with_raw_response.retrieve(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        monitor = response.parse()
        assert_matches_type(MonitorRetrieveResponse, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: NextbillionSDK) -> None:
        with client.skynet.monitor.with_streaming_response.retrieve(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            monitor = response.parse()
            assert_matches_type(MonitorRetrieveResponse, monitor, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.skynet.monitor.with_raw_response.retrieve(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: NextbillionSDK) -> None:
        monitor = client.skynet.monitor.update(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(SimpleResp, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: NextbillionSDK) -> None:
        monitor = client.skynet.monitor.update(
            id="id",
            key="key=API_KEY",
            description="description",
            geofence_config={"geofence_ids": ["string"]},
            geofence_ids=["string"],
            idle_config={
                "distance_tolerance": 0,
                "time_tolerance": 0,
            },
            match_filter={
                "include_all_of_attributes": '{\n  "asset_type": "delivery",\n  "area": "Los Angeles downtown"\n}',
                "include_any_of_attributes": '{\n  "asset_type": "delivery",\n  "area": "Los Angeles downtown"\n}',
            },
            meta_data={},
            name='"name":"warehouse_exit"',
            speeding_config={
                "customer_speed_limit": '"customer_speed_limit":8',
                "time_tolerance": 0,
                "use_admin_speed_limit": True,
            },
            tags=["string"],
            type="`enter`",
        )
        assert_matches_type(SimpleResp, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: NextbillionSDK) -> None:
        response = client.skynet.monitor.with_raw_response.update(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        monitor = response.parse()
        assert_matches_type(SimpleResp, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: NextbillionSDK) -> None:
        with client.skynet.monitor.with_streaming_response.update(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            monitor = response.parse()
            assert_matches_type(SimpleResp, monitor, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.skynet.monitor.with_raw_response.update(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: NextbillionSDK) -> None:
        monitor = client.skynet.monitor.list(
            key="key=API_KEY",
        )
        assert_matches_type(MonitorListResponse, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: NextbillionSDK) -> None:
        monitor = client.skynet.monitor.list(
            key="key=API_KEY",
            cluster="america",
            pn=0,
            ps=100,
            sort="updated_at:desc",
            tags="tags=tag_1,tag_2",
        )
        assert_matches_type(MonitorListResponse, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: NextbillionSDK) -> None:
        response = client.skynet.monitor.with_raw_response.list(
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        monitor = response.parse()
        assert_matches_type(MonitorListResponse, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: NextbillionSDK) -> None:
        with client.skynet.monitor.with_streaming_response.list(
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            monitor = response.parse()
            assert_matches_type(MonitorListResponse, monitor, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: NextbillionSDK) -> None:
        monitor = client.skynet.monitor.delete(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(SimpleResp, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: NextbillionSDK) -> None:
        response = client.skynet.monitor.with_raw_response.delete(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        monitor = response.parse()
        assert_matches_type(SimpleResp, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: NextbillionSDK) -> None:
        with client.skynet.monitor.with_streaming_response.delete(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            monitor = response.parse()
            assert_matches_type(SimpleResp, monitor, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_delete(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.skynet.monitor.with_raw_response.delete(
                id="",
                key="key=API_KEY",
            )


class TestAsyncMonitor:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncNextbillionSDK) -> None:
        monitor = await async_client.skynet.monitor.create(
            key="key=API_KEY",
            tags=["string"],
            type="`enter`",
        )
        assert_matches_type(MonitorCreateResponse, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        monitor = await async_client.skynet.monitor.create(
            key="key=API_KEY",
            tags=["string"],
            type="`enter`",
            cluster="america",
            custom_id="custom_id",
            description="description",
            geofence_config={"geofence_ids": ["string"]},
            geofence_ids=["string"],
            idle_config={
                "distance_tolerance": 0,
                "time_tolerance": 0,
            },
            match_filter={
                "include_all_of_attributes": '{\n  "asset_type": "delivery",\n  "area": "Los Angeles downtown"\n}',
                "include_any_of_attributes": '{\n  "asset_type": "delivery",\n  "area": "Los Angeles downtown"\n}',
            },
            meta_data={},
            name="name",
            speeding_config={
                "customer_speed_limit": 0,
                "time_tolerance": 0,
                "use_admin_speed_limit": True,
            },
        )
        assert_matches_type(MonitorCreateResponse, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.monitor.with_raw_response.create(
            key="key=API_KEY",
            tags=["string"],
            type="`enter`",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        monitor = await response.parse()
        assert_matches_type(MonitorCreateResponse, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.monitor.with_streaming_response.create(
            key="key=API_KEY",
            tags=["string"],
            type="`enter`",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            monitor = await response.parse()
            assert_matches_type(MonitorCreateResponse, monitor, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        monitor = await async_client.skynet.monitor.retrieve(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(MonitorRetrieveResponse, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.monitor.with_raw_response.retrieve(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        monitor = await response.parse()
        assert_matches_type(MonitorRetrieveResponse, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.monitor.with_streaming_response.retrieve(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            monitor = await response.parse()
            assert_matches_type(MonitorRetrieveResponse, monitor, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.skynet.monitor.with_raw_response.retrieve(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncNextbillionSDK) -> None:
        monitor = await async_client.skynet.monitor.update(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(SimpleResp, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        monitor = await async_client.skynet.monitor.update(
            id="id",
            key="key=API_KEY",
            description="description",
            geofence_config={"geofence_ids": ["string"]},
            geofence_ids=["string"],
            idle_config={
                "distance_tolerance": 0,
                "time_tolerance": 0,
            },
            match_filter={
                "include_all_of_attributes": '{\n  "asset_type": "delivery",\n  "area": "Los Angeles downtown"\n}',
                "include_any_of_attributes": '{\n  "asset_type": "delivery",\n  "area": "Los Angeles downtown"\n}',
            },
            meta_data={},
            name='"name":"warehouse_exit"',
            speeding_config={
                "customer_speed_limit": '"customer_speed_limit":8',
                "time_tolerance": 0,
                "use_admin_speed_limit": True,
            },
            tags=["string"],
            type="`enter`",
        )
        assert_matches_type(SimpleResp, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.monitor.with_raw_response.update(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        monitor = await response.parse()
        assert_matches_type(SimpleResp, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.monitor.with_streaming_response.update(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            monitor = await response.parse()
            assert_matches_type(SimpleResp, monitor, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.skynet.monitor.with_raw_response.update(
                id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncNextbillionSDK) -> None:
        monitor = await async_client.skynet.monitor.list(
            key="key=API_KEY",
        )
        assert_matches_type(MonitorListResponse, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        monitor = await async_client.skynet.monitor.list(
            key="key=API_KEY",
            cluster="america",
            pn=0,
            ps=100,
            sort="updated_at:desc",
            tags="tags=tag_1,tag_2",
        )
        assert_matches_type(MonitorListResponse, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.monitor.with_raw_response.list(
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        monitor = await response.parse()
        assert_matches_type(MonitorListResponse, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.monitor.with_streaming_response.list(
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            monitor = await response.parse()
            assert_matches_type(MonitorListResponse, monitor, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncNextbillionSDK) -> None:
        monitor = await async_client.skynet.monitor.delete(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(SimpleResp, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.skynet.monitor.with_raw_response.delete(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        monitor = await response.parse()
        assert_matches_type(SimpleResp, monitor, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.skynet.monitor.with_streaming_response.delete(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            monitor = await response.parse()
            assert_matches_type(SimpleResp, monitor, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.skynet.monitor.with_raw_response.delete(
                id="",
                key="key=API_KEY",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .v2 import (
    V2Resource,
    AsyncV2Resource,
    V2ResourceWithRawResponse,
    AsyncV2ResourceWithRawResponse,
    V2ResourceWithStreamingResponse,
    AsyncV2ResourceWithStreamingResponse,
)
from .optimization import (
    OptimizationResource,
    AsyncOptimizationResource,
    OptimizationResourceWithRawResponse,
    AsyncOptimizationResourceWithRawResponse,
    OptimizationResourceWithStreamingResponse,
    AsyncOptimizationResourceWithStreamingResponse,
)
from .driver_assignment import (
    DriverAssignmentResource,
    AsyncDriverAssignmentResource,
    DriverAssignmentResourceWithRawResponse,
    AsyncDriverAssignmentResourceWithRawResponse,
    DriverAssignmentResourceWithStreamingResponse,
    AsyncDriverAssignmentResourceWithStreamingResponse,
)

__all__ = [
    "DriverAssignmentResource",
    "AsyncDriverAssignmentResource",
    "DriverAssignmentResourceWithRawResponse",
    "AsyncDriverAssignmentResourceWithRawResponse",
    "DriverAssignmentResourceWithStreamingResponse",
    "AsyncDriverAssignmentResourceWithStreamingResponse",
    "V2Resource",
    "AsyncV2Resource",
    "V2ResourceWithRawResponse",
    "AsyncV2ResourceWithRawResponse",
    "V2ResourceWithStreamingResponse",
    "AsyncV2ResourceWithStreamingResponse",
    "OptimizationResource",
    "AsyncOptimizationResource",
    "OptimizationResourceWithRawResponse",
    "AsyncOptimizationResourceWithRawResponse",
    "OptimizationResourceWithStreamingResponse",
    "AsyncOptimizationResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["JobParam", "Volume"]


class Volume(TypedDict, total=False):
    alignment: Literal["`strict`", "`parallel`", "`fixed_bottom`", '`" "`']
    """Refers to the orientation of the cargo in the loading compartment.

    It supports the following values:

    - **" "** : A blank space to indicate that the dimension check for the cargo
      should be skipped. This is also the default value.
    - **strict :** In this orientation, the cargo must fit within the vehicle’s
      dimensions exactly as is—no rotation is allowed. All dimensions of the cargo
      must be less than or equal to the corresponding dimensions of the vehicle. If
      any dimension exceeds that of the vehicle, the cargo cannot be loaded.
    - **parallel :** With parallel orientation, the cargo can be rotated around any
      one of the axes to help it fit into the loading compartment. For example, if
      the cargo is wider than the vehicle, it can still be loaded by rotating it
      around the vertical axis (so the width aligns with the vehicle’s depth) or the
      horizontal axis (so the width aligns with the vehicle’s height). Using this
      orientation, even a cargo whose one dimension is larger than the corresponding
      dimension of the vehicle but smaller than other dimensions, can also be
      loaded.
    - **fixed_bottom :** In this orientation, the cargo can only be rotated around
      the vertical axis, meaning its base stays fixed and it cannot be tilted or
      turned over. The height of the cargo remains aligned with the vehicle’s height
      and cannot be adjusted. As a result, if the cargo’s height exceeds the
      vehicle’s available height, it cannot be loaded into the compartment.
    """

    depth: float
    """Cargo length, in meters."""

    height: float
    """Cargo height, in meters."""

    width: float
    """Cargo width, in meters."""


class JobParam(TypedDict, total=False):
    id: Required[str]
    """Provide an unique ID for the job. The IDs are case-sensitive."""

    location_index: Required[int]
    """
    An integer denoting the index (in the `location` array) of the location
    coordinates where the job needs to be performed. The valid range of values is
    \\[[0, length of `location` array).

    Please note the `location_index` is mandatory when using the `jobs` object.
    """

    delivery: Iterable[int]
    """
    In case the job involves a delivery step, use this attribute to describe
    delivery quantity. This attribute supports multidimensional quantities, to
    support delivering quantities of different units/dimensions. It is recommended
    to keep the dimensions of quantity for pickup/delivery consistent when defining
    them in `jobs` and `vehicles` (in `capacity` attribute).

    Please note that the quantity of delivery will be added to the assigned
    vehicle’s initial load.

    In case depots are being added, the delivery configured here can be fulfilled by
    vehicles starting from specific depots. Refer to `depot_ids` and
    `vehicles.start_depot_ids` to know more.
    """

    depot_ids: List[str]
    """Specify the depots which can be used to fulfil this job.

    In case of a pickup job, the assigned vehicle will deliver the goods to the
    depot specified here, at the end of its trip. Conversely, in case of delivery
    jobs, the vehicle will load the goods from the depot specified here, before
    starting the trip.

    **Note:**

    - If multiple IDs are provided for this field then any of the given depots can
      be used to fulfil the job.
    - If this field is not provided then the given job can be fulfilled by any
      vehicle (subject to other constraints configured in the input).
    - If the job does not have any delivery or pick-up step configured then depots
      configuration will not have any impact on the given job.
    """

    description: str
    """Add a custom description for the job."""

    follow_lifo_order: bool
    """Specify whether the job route should follow LIFO (last in, first out).

    Use this parameter when `pickup` or `delivery` jobs are involved and the loading
    or unloading sequence of cargo is important. The default is `false`.
    """

    incompatible_load_types: List[str]
    """
    Use this parameter to specify the type of loads which are incompatible with the
    job’s load type. Once this property is configured, the job can only be serviced
    by a vehicle which has not serviced any other task with an incompatible
    `load_types` . Add multiple load types to indicate all the types which are
    incompatible for this job. The incompatible load type considerations are ignored
    for the first task of the route.

    For example, an input value of \\[[“groceries”, “food”\\]] means that current job’s
    load is incompatible with both groceries and food type of loads. Consequently,
    the optimizer will not assign this job to a vehicle which has served any task
    with `load_types` as either groceries or food.

    Note:

    - This parameter is effective only when a pickup / delivery is configured for
      the job.
    - If the job is part of any `relations` then, configured
      `incompatible_load_types` might be ignored.
    """

    joint_order: int
    """Specify a joint order group ID that this job is associated with.

    Tasks having the same joint order group ID are treated as a single unit: either
    all tasks in the group are assigned, or none are. Users can add tasks of both
    `jobs` and `shipments` types to a single joint order group by using the same
    unique ID. Please note that:

    - Each job in a single joint order group will be fulfilled by a unique vehicle.
    - Jobs belonging to a joint order group can be served in any sequence.
    - Joint order settings will not be effective if \\``solution\\`` or \\``relations\\``
      attributes are also provided in the input request.
    """

    load_types: List[str]
    """Use this parameter to specify the type of loads for the given job.

    Once this property is configured, the job can not be served by a vehicle which
    has serviced any task whose load is incompatible with any of the`load_types`
    provided in this input. The load type considerations are ignored for the first
    task of the route.

    For example, an input value of \\[[“groceries”, “food”\\]] means that job’s load
    characteristics belong to either one or both types. Consequently, the optimizer
    will assign this job to a vehicle which has served other tasks whose
    `incompatible_load_types` do not contain either groceries or food.

    Note:

    - This parameter is effective only when a pickup / delivery is configured for
      the job.
    - If the job is part of any `relations` then, `load_types` might be ignored.
    """

    max_visit_lateness: int
    """
    It determines the allowable delay, in seconds, to begin a job after its
    designated time window has concluded. Please note that this parameter would
    override the `constraint.max_visit_lateness` (global) if both are specified.
    """

    metadata: object
    """
    Specify any custom data that should be attached along with job fulfilment
    details in the `steps` attribute of the optimized solution. Users can leverage
    this property to provide additional details/context when sharing information
    about the job with integrated systems (TMS, Fleet Management, Driver dispatch
    etc).

    Please note that the `metadata` content must always be specified in a`key` :
    `value` pair format, where the “key” is always a string.
    """

    outsourcing_cost: int
    """
    Specify the cost of keeping this job unassigned, namely, the cost of outsourcing
    the job. When provided, the optimizer weighs the cost of assigning the job
    against the cost of keeping it unassigned and chooses a solution with lower
    cost. In the solution, the `outsourcing_cost` of unassigned jobs is added to the
    total cost of the solution.

    If the `outsourcing_cost` is not provided, which is to say that the job can not
    be outsourced, then the optimizer tries to fulfill the job irrespective of the
    cost incurred, subject to other constraints.

    Please note that `revenue` and `outsourcing_cost` can not be specified
    simultaneously for a job. Also, the `outsourcing_cost` would override the
    priority settings of the job.
    """

    pickup: Iterable[int]
    """
    In case the job involves a pickup step, use this attribute to describe pickup
    quantity. This attribute supports multidimensional quantities, to support
    picking up quantities of different units/dimensions. It is recommended to keep
    the dimensions of quantity for pickup/delivery consistent when defining them in
    `jobs` and `vehicles` (in `capacity` attribute).

    Please note that the vehicle will continue to carry the picked-up quantity until
    its last stop.

    In case depots are being added, the pickup configured here can be fulfilled by
    vehicles ending at specific depots. Refer to `depot_ids` and
    `vehicles.end_depot_ids` to know more.
    """

    priority: int
    """Specify the priority of this job.

    The valid values are in the range of \\[[0, 100\\]]. Default value is 0.

    Please note that setting a priority will only decide whether this job will be
    assigned or not, but has nothing to do with the sequence of job fulfilment.
    """

    revenue: int
    """Specify the revenue earned by completing this job.

    The optimizer uses the `revenue` input to identify the potential profit earned
    by fulfilling this job after taking into account the costs incurred to do so.
    The`options.objective` and `vehicles.costs` input are taken into account to
    identify the costs of fulfilling the job.

    In general, the optimizer will prefer fulfilling the tasks with higher profits
    over the tasks with lower profits, should it need to reject some tasks in order
    to honor other constraints. In case the profit from fulfilling a job is
    negative, it will remain unassigned whatsoever.
    """

    sequence_order: int
    """
    Use this parameter to prioritize completing a task relative to certain other
    tasks. A task configured with a `sequence_order` of 2 will be done after the
    task with `sequence_order` of 1, but before the task with `sequence_order` of 3.
    Valid range of values for this input is \\[[0,100\\]].

    Please note that:

    - Only the tasks within the same route are compared and ordered as per their
      `sequence_order`.
    - Tasks without a `sequence_order` are not involved in the comparison.
    - Following is the precedence of `sequence_order` when used along side some of
      the other constraints:

      - `relations` are prioritized over `sequence_order` comparisons.
      - `sequence_order` will override `order_grouping` configurations.
    """

    service: int
    """
    Use this attribute to define the time duration, in seconds, needed to complete
    the job. Default value is 0.
    """

    setup: int
    """Specify the job set-up duration, in seconds.

    `setup` is the one-time effort needed apart from working on the original task -
    for example, effort to record some information for compliance, or effort to
    set-up the equipment, or perform any other action for completing all steps
    required to fulfil the job.

    Please note that `setup` time is applied only once for a given task location.
    `setup` time, unlike `service` time, is not repeated in case there are multiple
    tasks at the same location.
    """

    skills: Iterable[int]
    """Define the skills needed to complete the job.

    This attribute supports multidimensional skills allowing users to add multiple
    skills.

    Read more about the behavior of this attribute in the
    [Multi-Dimensional Parameters](#multi-dimensional-parameters) section.
    """

    time_windows: Iterable[Iterable[int]]
    """Define time periods within which this job should be started.

    The time periods should be expressed as a UNIX timestamp in seconds.

    Please note that the time periods should not overlap with each other and should
    always follow the format of \\[[start_timestamp, end_timestamp\\]].
    """

    volume: Volume
    """
    Specify the dimensions and alignment configurations for the cargo associated
    with the task. These inputs will be used to arrange the items into the loading
    compartment of the vehicle to utilize the three-dimensional space. If a job
    consists of several different items, each with its own dimensions, please
    specify the final characteristics for the task: total height, total length,
    total width.

    Please note that vehicles which contain the `volume` input, will only be
    considered for arranging such items.
    """

    zones: Iterable[int]
    """An array of integers specifying the IDs of the zone(s) that this job belongs to.

    The job can be fulfilled by all vehicles which are allowed to complete tasks in
    the zone(s) assigned to this job. Please note following points about job zones:

    - If zone IDs are provided for any one of the jobs, then all other jobs should
      also be specified with zone IDs. Zone IDs provided here will override any zone
      geometries provided in the `zones` attribute and these IDs will be used for
      allocating appropriate vehicles.
    - Jobs can be auto-allocated to zones if this parameter is not specified while
      the zone geometries (either `zones.geometry` or `zones.geofence_id`) are
      provided.
    - Jobs not falling in any zones can be fulfilled by only those vehicles which
      are allowed to take up tasks outside zones as well. Refer to `vehicles`
      attribute for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .geofence import Geofence as Geofence
from .polygon_geojson import PolygonGeojson as PolygonGeojson
from .batch_list_params import BatchListParams as BatchListParams
from .batch_create_params import BatchCreateParams as BatchCreateParams
from .batch_delete_params import BatchDeleteParams as BatchDeleteParams
from .batch_list_response import BatchListResponse as BatchListResponse
from .batch_create_response import BatchCreateResponse as BatchCreateResponse
from .console_search_params import ConsoleSearchParams as ConsoleSearchParams
from .console_preview_params import ConsolePreviewParams as ConsolePreviewParams
from .console_search_response import ConsoleSearchResponse as ConsoleSearchResponse
from .console_preview_response import ConsolePreviewResponse as ConsolePreviewResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

from .route_step_completion_mode import RouteStepCompletionMode
from .route_step_geofence_config_param import RouteStepGeofenceConfigParam

__all__ = ["RouteStepsRequestParam", "Meta"]


class Meta(TypedDict, total=False):
    customer_name: str
    """Specify the name of the customer for which the step has to be performed."""

    customer_phone_number: str
    """Specify the phone number of the person to be contacted when at step location."""

    instructions: str
    """Specify custom instructions to be carried out while performing the step."""


class RouteStepsRequestParam(TypedDict, total=False):
    arrival: Required[int]
    """
    Specify the scheduled arrival time of the driver, as an UNIX timestamp in
    seconds, at the step. Please note that:

    - Arrival time for each step should be equal to or greater than the previous
      step.
    - Past times can not be provided.
    - The time provided is used only for informative display on the driver app and
      it does not impact or get affected by the route generated.
    """

    location: Required[Iterable[float]]
    """
    Specify the location coordinates where the steps should be performed in
    `[latitude, longitude]`.
    """

    type: Required[Literal["`start`", "`job`", "`pickup`", "`delivery`", "`break`", "`layover`", "`end`"]]
    """Specify the step type.

    It can belong to one of the following: `start`, `job` , `pickup`, `delivery`,
    `end`. A `duration` is mandatory when the step type is either `layover` or a
    `break`.
    """

    address: str
    """Specify the postal address for the step."""

    completion_mode: RouteStepCompletionMode
    """Specify the mode of completion to be used for the step.

    Currently, following values are allowed:

    - `manual`: Steps must be marked as completed manually through the Driver App.
    - `geofence`: Steps are marked as completed automatically based on the entry
      conditions and geofence specified.
    - `geofence_manual_fallback`: Steps will be marked as completed automatically
      based on geofence and entry condition configurations but there will also be a
      provision for manually updating the status in case, geofence detection fails.
    """

    document_template_id: str
    """
    Specify the ID of the document template to be used for collecting proof of
    completion for the step. If not specified, the document template specified at
    the route level will be used for the step. Use the
    [Documents API](https://docs.nextbillion.ai/docs/dispatches/documents-api) to
    create, read and manage the document templates.

    Please note that the document template ID can not be assigned to following step
    types - `start`, `end`, `break`, `layover`.
    """

    duration: int
    """Specify the duration of the `layover` or `break` type steps, in seconds.

    Please note it is mandatory when step type is either "layover" or "break".
    """

    geofence_config: RouteStepGeofenceConfigParam
    """
    Specify the configurations of the geofence which will be used to detect presence
    of the driver and complete the tasks automatically. Please note that this
    attribute is required when `completion_mode` is either "geofence" or
    "geofence_manual_fallback".
    """

    meta: Meta
    """
    An object to specify any additional details about the task to be associated with
    the step in the response. The information provided here will be available on the
    Driver's app under step details. This attribute can be used to provide context
    about or instructions to the driver for performing the task
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ConfigUpdateParams"]


class ConfigUpdateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    cluster: Literal["america"]
    """the cluster of the region you want to use"""

    webhook: List[str]
    """Use this array to update information about the webhooks.

    Please note that the webhooks will be overwritten every time this method is
    used.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["PlaceCreateParams", "Place", "PlaceGeopoint", "PlacePoi", "DataSource"]


class PlaceCreateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    place: Required[Iterable[Place]]
    """
    This parameter represents the place details, including geographical information,
    address and other related information.
    """

    data_source: Annotated[DataSource, PropertyInfo(alias="dataSource")]
    """It contains information about the dataset that returns the specific result"""

    force: bool
    """
    When 2 places are located within 100 meters of each other and have more than 90%
    of matching attributes (at least 11 out of 12 attributes in the “place” object),
    they will be considered duplicates and any requests to add such a new place
    would be rejected. Set `force=true` to override this duplicate check. You can
    use this to create closely located POIs. For instance, places inside a mall,
    university or a government building etc.
    """

    score: int
    """Search score of the place.

    This is calculated based on how ‘richly’ the place is defined. For instance, a
    place with - street name, city, state and country attributes set might be ranked
    lower than a place which has values of - house, building, street name, city,
    state and country attributes set. The score determines the rank of the place
    among search results. You can also use this field to set a custom score as per
    its relevance to rank it among the search results from multiple data sources.
    """


class PlaceGeopoint(TypedDict, total=False):
    lat: float
    """This parameter represents the latitude value of the place."""

    lng: float
    """This parameter represents the longitude value of the place."""


class PlacePoi(TypedDict, total=False):
    title: str
    """A title that describes the point of interest."""


class Place(TypedDict, total=False):
    geopoint: Required[PlaceGeopoint]
    """This parameter represents the geographical coordinates of the place.

    It includes the latitude and longitude values.
    """

    address: str
    """
    This parameter represents the complete address of the place, including the
    street, city, state, postal code and country.
    """

    building: str
    """This parameter represents additional building information if applicable."""

    city: str
    """This parameter represents the city or town of the place."""

    country: str
    """
    Country of the search context provided as comma-separated
    [ISO 3166-1 alpha-3](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-3) country
    codes.

    Note: Country codes should be provided in uppercase.
    """

    district: str
    """This parameter represents the district of the place."""

    house: str
    """This parameter represents the house or building number of the place."""

    poi: PlacePoi
    """This parameter represents a point of interest within the place.

    A Point of Interest (POI) refers to a specific location or area that is of
    interest to individuals for various reasons. It could be a landmark, tourist
    attraction, business, or any other location that people might find important or
    intriguing.
    """

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]
    """This parameter represents the postal code or ZIP code of the place."""

    state: str
    """This parameter represents the state or region of the place."""

    street: str
    """This parameter represents the street name of the place."""

    sub_district: Annotated[str, PropertyInfo(alias="subDistrict")]
    """This parameter represents the sub-district or locality of the place."""


class DataSource(TypedDict, total=False):
    ref_id: Annotated[str, PropertyInfo(alias="refId")]
    """
    This parameter represents the unique reference ID associated with the data
    source.
    """

    source: str
    """This parameter represents the source of the data."""

    status: Literal["enable", "disable"]
    """This parameter indicates if a place is searchable."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

__all__ = ["RestrictionListByBboxParams"]


class RestrictionListByBboxParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    max_lat: Required[float]
    """Specifies the maximum latitude value for the bounding box."""

    max_lon: Required[float]
    """Specifies the maximum longitude value for the bounding box."""

    min_lat: Required[float]
    """Specifies the minimum latitude value for the bounding box."""

    min_lon: Required[float]
    """Specifies the minimum longitude value for the bounding box."""

    mode: List[Literal["0w", "2w", "3w", "4w", "6w"]]
    """Specify the modes of travel that the restriction pertains to."""

    restriction_type: Literal["turn", "parking", "fixedspeed", "maxspeed", "closure", "truck"]
    """Specify the type of restrictions to fetch."""

    source: Literal["rrt", "pbf"]
    """
    This parameter represents where the restriction comes from and cannot be
    modified by clients sending requests to the API endpoint.

    For example, an API endpoint that returns a list of restrictions could include
    the source parameter to indicate where each item comes from. This parameter can
    be useful for filtering, sorting, or grouping the results based on their source.
    """

    state: Literal["`enabled`", "`disabled`", "`deleted`"]
    """This parameter is used to filter restrictions based on their state i.e.

    whether the restriction is currently enabled, disabled, or deleted. For example,
    users can retrieve a list of all the deleted restrictions by setting
    `state=deleted`.
    """

    status: Literal["`active`", "`inactive`"]
    """Restrictions can be active or inactive at a given time by virtue of their
    nature.

    For example, maximum speed limits can be active on the roads leading to schools
    during school hours and be inactive afterwards or certain road closure
    restrictions be active during holidays/concerts and be inactive otherwise.

    Use this parameter to filter the restrictions based on their status at the time
    of making the request i.e. whether they are in force or not.
    """

    transform: bool
    """This is internal parameter with a default value as `false`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types.distancematrix import JsonRetrieveResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestJson:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: NextbillionSDK) -> None:
        json = client.distancematrix.json.create()
        assert json is None

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: NextbillionSDK) -> None:
        response = client.distancematrix.json.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        json = response.parse()
        assert json is None

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: NextbillionSDK) -> None:
        with client.distancematrix.json.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            json = response.parse()
            assert json is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: NextbillionSDK) -> None:
        json = client.distancematrix.json.retrieve(
            destinations="destinations=41.349302,2.136480|41.389925,2.136258|41.357961,2.097878",
            key="key=API_KEY",
            origins="origins:41.349302,2.136480|41.389925,2.136258|41.357961,2.097878",
        )
        assert_matches_type(JsonRetrieveResponse, json, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_with_all_params(self, client: NextbillionSDK) -> None:
        json = client.distancematrix.json.retrieve(
            destinations="destinations=41.349302,2.136480|41.389925,2.136258|41.357961,2.097878",
            key="key=API_KEY",
            origins="origins:41.349302,2.136480|41.389925,2.136258|41.357961,2.097878",
            approaches="`unrestricted`",
            avoid="toll",
            bearings="bearings=0,180;0,180",
            mode="`car`",
            route_failed_prompt=True,
        )
        assert_matches_type(JsonRetrieveResponse, json, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: NextbillionSDK) -> None:
        response = client.distancematrix.json.with_raw_response.retrieve(
            destinations="destinations=41.349302,2.136480|41.389925,2.136258|41.357961,2.097878",
            key="key=API_KEY",
            origins="origins:41.349302,2.136480|41.389925,2.136258|41.357961,2.097878",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        json = response.parse()
        assert_matches_type(JsonRetrieveResponse, json, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: NextbillionSDK) -> None:
        with client.distancematrix.json.with_streaming_response.retrieve(
            destinations="destinations=41.349302,2.136480|41.389925,2.136258|41.357961,2.097878",
            key="key=API_KEY",
            origins="origins:41.349302,2.136480|41.389925,2.136258|41.357961,2.097878",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            json = response.parse()
            assert_matches_type(JsonRetrieveResponse, json, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncJson:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncNextbillionSDK) -> None:
        json = await async_client.distancematrix.json.create()
        assert json is None

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.distancematrix.json.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        json = await response.parse()
        assert json is None

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.distancematrix.json.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            json = await response.parse()
            assert json is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        json = await async_client.distancematrix.json.retrieve(
            destinations="destinations=41.349302,2.136480|41.389925,2.136258|41.357961,2.097878",
            key="key=API_KEY",
            origins="origins:41.349302,2.136480|41.389925,2.136258|41.357961,2.097878",
        )
        assert_matches_type(JsonRetrieveResponse, json, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        json = await async_client.distancematrix.json.retrieve(
            destinations="destinations=41.349302,2.136480|41.389925,2.136258|41.357961,2.097878",
            key="key=API_KEY",
            origins="origins:41.349302,2.136480|41.389925,2.136258|41.357961,2.097878",
            approaches="`unrestricted`",
            avoid="toll",
            bearings="bearings=0,180;0,180",
            mode="`car`",
            route_failed_prompt=True,
        )
        assert_matches_type(JsonRetrieveResponse, json, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.distancematrix.json.with_raw_response.retrieve(
            destinations="destinations=41.349302,2.136480|41.389925,2.136258|41.357961,2.097878",
            key="key=API_KEY",
            origins="origins:41.349302,2.136480|41.389925,2.136258|41.357961,2.097878",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        json = await response.parse()
        assert_matches_type(JsonRetrieveResponse, json, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.distancematrix.json.with_streaming_response.retrieve(
            destinations="destinations=41.349302,2.136480|41.389925,2.136258|41.357961,2.097878",
            key="key=API_KEY",
            origins="origins:41.349302,2.136480|41.389925,2.136258|41.357961,2.097878",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            json = await response.parse()
            assert_matches_type(JsonRetrieveResponse, json, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["RestrictionUpdateParams", "Segment", "Turn"]


class RestrictionUpdateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    area: Required[str]
    """Specify the area name.

    It represents a region where restrictions can be applied. This is a custom field
    and it is recommended for the users to check with
    [NextBillion.ai](www.nextbillion.ai) support for the right value. Alternatively,
    users can invoke the _[Areas](#supported-areas)_ method to get a list of
    available areas for them.
    """

    name: Required[str]
    """Specify a custom, descriptive name for the restriction."""

    latlon: bool
    """Use this parameter to decide the format for specifying the geofence coordinates.

    If true, then the coordinates of geofence can be specified as
    "latitude,longitude" format, otherwise they should be specified in
    "longitude,latitude" format.
    """

    comment: str
    """
    Use this parameter to add any custom information about the restriction being
    created.
    """

    direction: Literal["`forward`", "`backward`", "`both`"]
    """
    Represents the traffic direction on the segments to which the restriction will
    be applied.
    """

    end_time: float
    """
    Provide a UNIX epoch timestamp in seconds, representing the time when the
    restriction should cease to be in-effect.
    """

    geofence: Iterable[Iterable[float]]
    """
    An array of coordinates denoting the boundary of an area in which the
    restrictions are to be applied. The format in which coordinates should be listed
    is defined by the `latlon` field.

    Geofences can be used to create all restriction types, except for a `turn` type
    restriction. Please note that `segments` is not required when using `geofence`
    to create restrictions.
    """

    height: int
    """
    Specify the maximum truck height, in centimeter, that will be allowed under the
    restriction. A value of 0 indicates no limit.

    Please note this parameter is effective only when `restriction_type` is `truck`.
    At least one of truck parameters - `weight`, `height`, `width` and `truck` -
    needs to be provided when restriction type is `truck`.
    """

    length: int
    """
    Specify the maximum truck length, in centimeter, that will be allowed under the
    restriction. A value of 0 indicates no limit.

    Please note this parameter is effective only when `restriction_type` is `truck`.
    At least one of truck parameters - `weight`, `height`, `width` and `truck` -
    needs to be provided when restriction type is `truck`.
    """

    mode: List[Literal["0w", "2w", "3w", "4w", "6w"]]
    """Provide the driving modes for which the restriction should be effective.

    If the value is an empty array or if it is not provided then the restriction
    would be applied for all modes.
    """

    repeat_on: str
    """It represents the days and times when the restriction is in effect.

    Users can use this property to set recurring or one-time restrictions as per the
    [given format](https://wiki.openstreetmap.org/wiki/Key:opening_hours) for
    specifying the recurring schedule of the restriction.

    Please provided values as per the local time of the region where the restriction
    is being applied.
    """

    segments: Iterable[Segment]
    """
    An array of objects to collect the details of the segments of a road on which
    the restriction has to be applied. Each object corresponds to a new segment.

    Please note that `segments` is mandatory for all `restrtiction_type` except
    `turn`.
    """

    speed: float
    """
    Provide the the fixed speed of the segment where the restriction needs to be
    applied. Please note that this parameter is mandatory when the `restrictionType`
    is `fixedspeed`.
    """

    speed_limit: float
    """
    Provide the the maximum speed of the segment where the restriction needs to be
    applied. Please note that this parameter is mandatory when the `restrictionType`
    is `maxspeed`.
    """

    start_time: float
    """
    Provide a UNIX epoch timestamp in seconds, representing the start time for the
    restriction to be in-effect.
    """

    tracks: Iterable[Iterable[float]]
    """Specify a sequence of coordinates (track) where the restriction is to be
    applied.

    The coordinates will be snapped to nearest road. Please note when using
    `tracks`, `segments` and `turns` are not required.
    """

    turns: Iterable[Turn]
    """
    An array of objects to collect the details of the turns of a road on which the
    restriction has to be applied. Each object corresponds to a new turn.

    Please note that `turns` is mandatory for when `restrtiction_type=turn`.
    """

    weight: int
    """Specify the maximum truck weight, in kilograms, that the restriction will allow.

    A value of 0 indicates no limit.

    Please note this parameter is effective only when `restriction_type` is `truck`.
    At least one of truck parameters - `weight`, `height`, `width` and `truck` -
    needs to be provided for is `truck` restriction type.
    """

    width: int
    """
    Specify the maximum truck width, in centimeter, that will be allowed under the
    restriction. A value of 0 indicates no limit.

    Please note this parameter is effective only when `restriction_type` is `truck`.
    At least one of truck parameters - `weight`, `height`, `width` and `truck` -
    needs to be provided when restriction type is `truck`.
    """


_SegmentReservedKeywords = TypedDict(
    "_SegmentReservedKeywords",
    {
        "from": float,
    },
    total=False,
)


class Segment(_SegmentReservedKeywords, total=False):
    to: float
    """An integer value representing the ID of the ending node of the segment."""


_TurnReservedKeywords = TypedDict(
    "_TurnReservedKeywords",
    {
        "from": int,
    },
    total=False,
)


class Turn(_TurnReservedKeywords, total=False):
    to: int
    """An integer value that represents the ID of the ending node of the turn."""

    via: int
    """
    An integer value that represents the ID of a node connecting `from` and `to`
    nodes of the turn.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .trip import (
    TripResource,
    AsyncTripResource,
    TripResourceWithRawResponse,
    AsyncTripResourceWithRawResponse,
    TripResourceWithStreamingResponse,
    AsyncTripResourceWithStreamingResponse,
)
from .asset import (
    AssetResource,
    AsyncAssetResource,
    AssetResourceWithRawResponse,
    AsyncAssetResourceWithRawResponse,
    AssetResourceWithStreamingResponse,
    AsyncAssetResourceWithStreamingResponse,
)
from .config import (
    ConfigResource,
    AsyncConfigResource,
    ConfigResourceWithRawResponse,
    AsyncConfigResourceWithRawResponse,
    ConfigResourceWithStreamingResponse,
    AsyncConfigResourceWithStreamingResponse,
)
from .search import (
    SearchResource,
    AsyncSearchResource,
    SearchResourceWithRawResponse,
    AsyncSearchResourceWithRawResponse,
    SearchResourceWithStreamingResponse,
    AsyncSearchResourceWithStreamingResponse,
)
from .skynet import (
    SkynetResource,
    AsyncSkynetResource,
    SkynetResourceWithRawResponse,
    AsyncSkynetResourceWithRawResponse,
    SkynetResourceWithStreamingResponse,
    AsyncSkynetResourceWithStreamingResponse,
)
from .monitor import (
    MonitorResource,
    AsyncMonitorResource,
    MonitorResourceWithRawResponse,
    AsyncMonitorResourceWithRawResponse,
    MonitorResourceWithStreamingResponse,
    AsyncMonitorResourceWithStreamingResponse,
)
from .namespaced_apikeys import (
    NamespacedApikeysResource,
    AsyncNamespacedApikeysResource,
    NamespacedApikeysResourceWithRawResponse,
    AsyncNamespacedApikeysResourceWithRawResponse,
    NamespacedApikeysResourceWithStreamingResponse,
    AsyncNamespacedApikeysResourceWithStreamingResponse,
)

__all__ = [
    "AssetResource",
    "AsyncAssetResource",
    "AssetResourceWithRawResponse",
    "AsyncAssetResourceWithRawResponse",
    "AssetResourceWithStreamingResponse",
    "AsyncAssetResourceWithStreamingResponse",
    "MonitorResource",
    "AsyncMonitorResource",
    "MonitorResourceWithRawResponse",
    "AsyncMonitorResourceWithRawResponse",
    "MonitorResourceWithStreamingResponse",
    "AsyncMonitorResourceWithStreamingResponse",
    "TripResource",
    "AsyncTripResource",
    "TripResourceWithRawResponse",
    "AsyncTripResourceWithRawResponse",
    "TripResourceWithStreamingResponse",
    "AsyncTripResourceWithStreamingResponse",
    "NamespacedApikeysResource",
    "AsyncNamespacedApikeysResource",
    "NamespacedApikeysResourceWithRawResponse",
    "AsyncNamespacedApikeysResourceWithRawResponse",
    "NamespacedApikeysResourceWithStreamingResponse",
    "AsyncNamespacedApikeysResourceWithStreamingResponse",
    "ConfigResource",
    "AsyncConfigResource",
    "ConfigResourceWithRawResponse",
    "AsyncConfigResourceWithRawResponse",
    "ConfigResourceWithStreamingResponse",
    "AsyncConfigResourceWithStreamingResponse",
    "SearchResource",
    "AsyncSearchResource",
    "SearchResourceWithRawResponse",
    "AsyncSearchResourceWithRawResponse",
    "SearchResourceWithStreamingResponse",
    "AsyncSearchResourceWithStreamingResponse",
    "SkynetResource",
    "AsyncSkynetResource",
    "SkynetResourceWithRawResponse",
    "AsyncSkynetResourceWithRawResponse",
    "SkynetResourceWithStreamingResponse",
    "AsyncSkynetResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["GeofenceCreateParams", "Circle", "CircleCenter", "Isochrone", "Polygon", "PolygonGeojson"]


class GeofenceCreateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    type: Required[Literal["`circle`", "`polygon`", "`isochrone`"]]
    """Specify the type of the geofence that is being created."""

    circle: Circle
    """Provide the details to create a circular geofence.

    Please note that this object is mandatory when `type` is `circle`. When the
    `type` is not `circle`, the properties of this object will be ignored while
    creating the geofence.
    """

    custom_id: str
    """Set an unique ID for the new geofence.

    If not provided, an ID will be automatically generated in UUID format. A valid
    `custom_id` can contain letters, numbers, "-", & "\\__" only.

    Please note that the ID of a geofence can not be changed once it is created.
    """

    isochrone: Isochrone
    """Provide the details to create an isochrone based geofence.

    Use this object when `type` is `isochrone`. When the `type` is not `isochrone`,
    the properties of this object will be ignored while creating the geofence.
    """

    meta_data: object
    """Metadata of the geofence.

    Use this field to define custom attributes that provide more context and
    information about the geofence being created like country, group ID etc.

    The data being added should be in valid JSON object format (i.e. `key` and
    `value` pairs). Max size allowed for the object is 65kb.
    """

    name: str
    """Name of the geofence.

    Use this field to assign a meaningful, custom name to the geofence being
    created.
    """

    polygon: Polygon
    """Provide the details to create a custom polygon type of geofence.

    Please note that this object is mandatory when `type` is `polygon`. When the
    `type` is not `polygon`, the properties of this object will be ignored while
    creating the geofence.

    Self-intersecting polygons or polygons containing other polygons are invalid and
    will be removed while processing the request.

    Area of the polygon should be less than 2000 km<sup>2</sup>.
    """

    tags: List[str]
    """An array of strings to associate multiple tags to the geofence.

    `tags` can be used to search or filter geofences (using `Get Geofence List`
    method).

    Create valid `tags` using a string consisting of alphanumeric characters (A-Z,
    a-z, 0-9) along with the underscore ('\\__') and hyphen ('-') symbols.
    """


class CircleCenter(TypedDict, total=False):
    lat: Required[float]
    """Latitude of the `center` location."""

    lon: Required[float]
    """Longitude of the `center` location."""


class Circle(TypedDict, total=False):
    center: Required[CircleCenter]
    """Coordinate of the location which will act as the center of a circular geofence."""

    radius: Required[float]
    """Radius of the circular geofence, in meters.

    Maximum value allowed is 50000 meters.
    """


class Isochrone(TypedDict, total=False):
    coordinates: Required[str]
    """
    Coordinates of the location, in [latitude,longitude] format, which would act as
    the starting point for identifying the isochrone polygon or the boundary of
    reachable area. This parameter is mandatory when `type` is `isochrone`.
    """

    contours_meter: int
    """The distance, in meters, for which an isochrone polygon needs to be determined.

    When provided, the API would create a geofence representing the area that can be
    reached after driving the given number of meters starting from the point
    specified in `coordinates`.

    The maximum distance that can be specified is 60000 meters (60km).

    At least one of `contours_meter` or `contours_minute` is mandatory when `type`
    is `isochrone`.
    """

    contours_minute: int
    """The duration, in minutes, for which an isochrone polygon needs to be determined.

    When provided, the API would create a geofence representing the area that can be
    reached after driving for the given number of minutes starting from the point
    specified in `coordinates`.

    The maximum duration that can be specified is 40 minutes.

    At least one of `contours_meter` or `contours_minute` is mandatory when `type`
    is `isochrone`.
    """

    denoise: float
    """
    A floating point value from 0.0 to 1.0 that can be used to remove smaller
    contours. A value of 1.0 will only return the largest contour for a given value.
    A value of 0.5 drops any contours that are less than half the area of the
    largest contour in the set of contours for that same value.
    """

    departure_time: int
    """
    A UNIX epoch timestamp in seconds format that can be used to set the departure
    time. The isochrone boundary will be determined based on the typical traffic
    conditions at the given time. If no input is provided for this parameter then
    the traffic conditions at the time of making the request are considered
    """

    mode: Literal["`car`", "`truck`"]
    """Set which driving mode the service should use to determine the isochrone line.

    For example, if you use `car`, the API will return an isochrone polygon that a
    car can cover within the specified time or after driving the specified distance.
    Using `truck` will return an isochrone that a truck can reach after taking into
    account appropriate truck routing restrictions.
    """


class PolygonGeojson(TypedDict, total=False):
    coordinates: Required[Iterable[Iterable[float]]]
    """
    An array of coordinates in the [longitude, latitude] format, representing the
    geofence boundary.
    """

    type: Required[str]
    """Type of the geoJSON geometry. Should always be `Polygon`."""


class Polygon(TypedDict, total=False):
    geojson: Required[PolygonGeojson]
    """An object to collect geoJSON details of the geofence.

    The contents of this object follow the
    [geoJSON standard](https://datatracker.ietf.org/doc/html/rfc7946).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ConfigTestWebhookResponse"]


class ConfigTestWebhookResponse(BaseModel):
    status: Optional[str] = None
    """A string indicating the state of the response.

    Please note this value will always be `Ok`.

    The sample event information will be received on the webhook, if they were
    successfully configured. If no event information is received by the webhook,
    please reconfigure the webhook or else reach out to
    [support@nextbillion.ai](mailto:support@nextbillion.ai) for help.
    """
